##### Bootstrap Confidence Intervals #####
## R functions:
##     bootInterval: generate bootci object that gives bootstrap
##                   confidence intervals 
##     print.bootci: print method for bootci
##     check: check normality for bootstrap fample
##     summary.bootci: summary method for bootci
##########################################

"bootInterval" <-
function(x, y=NULL, m=length(x), n=length(y), one.sample=is.null(y),
         paired=FALSE, B=1000, parameter=c("mean","variance","proportion"),
         comparison=c("difference","ratio"), log.=FALSE,
	 conf.level=0.95, digits=3)
##########################################################################
#              Bootstrap Confidence Intervals
#
# Input:
#         x : Sample one
#         y : if its value is NULL (default), then it is one-sample
#             problem; otherwise, it is two-sample problem
#         m : length of x (sample size one)
#         n : length of y (sample size two)
#         one.sample : logical value indicating whether it is one-sample
#                      problem (TRUE) or not (FALSE, two-sample problem)
#         paired : logical value (default is FALSE) indicating whether
#                  it's paired-sample problem or not
#         B : bootstrap sample size (default is 1000)
#         parameter : one of "mean" (default), "variance", and
#                     "proportion".  Partial matching is possible.
#         comparison : one of "difference" (default) or "ratio".  It is
#                      in effect only if it is two-sample problem
#         log. : should logrithm be applied to the parameter, default
#                is FALSE
#         conf.level : confidence level of the interval, default is 0.95
#         digits : number of significance digits to label percentages
#                  of the confidence interval's end points
# Returned value : a list of class 'bootci' with the following components
#         parameter : parameter of interest
#         comparison: (only for two-sample problem) comparison type
#         one.sample: logical value indicating one-sample or not
#         paired    : paired-sample or not
#         data.     : name(s) of the sample(s)
#         log.      : logical value indicating whether or not logrithm
#                     was applied to parameter (and estimate)
#         boot.size : bootstrap size (i.e., number of bootstrap samples)
#         conf.level    : confidence level
#         estimate  : bootstrap estimate
#         se.boot   : standard error of estimate
#         perci     : percentile confidence interval
#         tci       : (currently only for one-sample problem) bootstrap
#                     confidence interval
#         sample.   : bootstrap sample (of length boot.size)
#
# Created by JC Wang, Jan. 15, 2008
#   1st modification (Jan. 24, 2008) :
##########################################################################
{parameter <- match.arg(parameter)
 one <- deparse(substitute(x))
 if (!one.sample)  # if two-sample
  two <- deparse(substitute(y))
 if (length(x)==1) {  # case for proportion for sample one
  if (missing(m)) stop("Missing sample size m!")
  if (x>m) stop("Count x is greater than sample size m!")
  x <- rep(1:0,c(x,m-x))
  one <- "x"
  parameter <- "proportion"
 }
 if (length(y)==1) {  # case for proportion for sample two
  if (missing(n)) stop("Missing sample size n!")
  if (y>n) stop("Count y is greater than sample size n!")
  y <- rep(1:0,c(y,n-y))
  two <- "y"
 }

 E <- if (parameter == "variance") var else mean

 comparison <- match.arg(comparison)
 if (parameter=="variance")
  comparison <- "ratio"

 if (paired) {
  if (m!=n)
   stop(paste("The samples",one,"and",two,"must be of the same size."))
  if (parameter!="mean")
   stop("Only difference in mean is implemented for paired-sample problem.")
  if (log.) {
   warning("Logrithm ignored for paired-sample problem.")
   log. <- FALSE
  }
  x <- x - y
  y <- NULL
  one.sample <- TRUE
 }

 estimate <- if (one.sample) {
               if (log.) function(a)log(E(a)) else E
	       }
              else if (comparison=="difference")
	       function(a,b) E(a) - E(b)
	      else if (log.)
	       function(a,b) log(E(a)) - log(E(b))
	      else
	       function(a,b) E(a) / E(b)

 if (log.) comparison <- "difference"

 bs <- numeric(B)
 if (one.sample)
   for(i in 1:B) {
    first <- sample(x, m, replace=T)
    bs[i] <- estimate(first)
   }
  else
   for(i in 1:B) {
    first <- sample(x, m, replace=T)
    second <- sample(y, n, replace=T)
    bs[i] <- estimate(first,second)
   }

 boot <- mean(bs)
 se.boot <- sd(bs) * sqrt((B-1) / B)

 a <- (1-conf.level) / 2
 a <- c(a, 1-a)
 pct <- stats:::format.perc(a, digits)

 index <- floor(a[1]*B);  index <- c(index, B-index)
 perci <- sort(bs)[index]
 names(perci) <- pct

 if (one.sample)
  {df. <- m - 1
   tci <- boot + qt(a, df.) * se.boot
   names(tci) <- pct
  }

 ans <-
  if (one.sample)
   list(parameter=parameter, one.sample=one.sample, paired=paired,
        data.=if (paired)   c(one,two)   else   one, log.=log.,
        boot.size=B, conf.level=conf.level, estimate=boot, se=se.boot,
	perci=perci, tci=tci, sample.=bs)
  else
   list(parameter=parameter, comparison=comparison, one.sample=one.sample,
        data.=c(one,two), log.=log., boot.size=B, conf.level=conf.level,
        estimate=boot, se=se.boot, perci=perci, sample.=bs)
 class(ans) <- "bootci"
 ans
}

"print.bootci" <-
function(obj)
##########################################################################
#   Print bootstrap confidence interval object
# Input
#       obj: an object of class 'bootci'
#
# Created by JC Wang, Jan. 15, 2008
##########################################################################
{i <- match("sample.", names(obj))
 print(obj[-i])
}

"check" <-
function(obj, method="Shapiro", plot.=TRUE, print.=!plot.,
         ylab="Bootstrap Sample", lty=3, lcol="red", digits=4, ...)
##########################################################################
#        Check normality of the bootstrap sample
#
# Input
#       obj    : an object of class 'bootci'
#       method : a character value indicating method for testing
#                normality, currently only "Shapiro" (Shapiro-Wilks
#                normality test) is available
#       plot.  : logical value (default is TRUE) indicating whether
#                or not normal Q-Q plot should be plotted
#       print. : logical value (default is !plot.) indicating whether
#                or not test result should be printed
#       ylab   : y-axis label (default is "Bootstrap Sample")
#       lty    : line type (default is 3) for Q-Q line
#       lcol   : line color (default is "red") for Q-Q line
#       digits : number of significant digits (default is 4) to display
#                test statistic and P value on the plot
#       ...    : other graphical parameters that can be passed to
#                plot function qqnorm
# Graphical output: Normal Q-Q plot with Q-Q line, test statistic
#                   and P value will be displayed on the plot
#
# Created by JC Wang, Jan. 15, 2008
##########################################################################
{method <- match.arg(method)
 data <- obj$sample.
 test. <- switch(method,
		 Shapiro=shapiro.test(data)
                 )
 test.$data.name <- "Bootstrap Sample"
 if (print.) print(test.)

 if (plot.)
  {stat <- signif(test.$statistic, digits)
   pval <- signif(test.$p.value, digits)
   title <- test.$method
   qqnorm(data, ylab=ylab, ...)
   qqline(data, lty=lty, col=lcol)
   w <- strwidth(title)
   h <- strheight(title)
   usr <- par('usr')
   x <- usr[1] + w/2 + strwidth("W")
   y <- usr[4] - h/2 - strheight("W")
   text(x,y, lab=title, font=3)
   text(x, y - 1.1*h, label=paste("Statistic", stat, sep=" = "))
   text(x, y - 2.2*h, label=paste("P Value", pval, sep=" = "))
  }
}

"summary.bootci" <-
function(obj, type=c("percentile", "t", "both"))
##########################################################################
#       Summary of a 'bootci' class object
#
# Input
#       obj  : an object of class 'bootci'
#       type : one of "percentile", "t", or "both" indicating type of
#              confidence intervals to be summarized.  Wherever
#              applicable, "both" calls for both bootstrap percentile
#              c.i., and bootstrap t c.i. be summarized. Partial matching
#              of type is possible.
# Output: Descriptive summary of confidence interval(s)
#
# Created by JC Wang, Jan. 15, 2008
##########################################################################
{type <- match.arg(type)
 percent <- type != "t"
 t. <- type != "percentile"
 t. <- t. && obj$one.sample

 title <- paste(ifelse(obj$one.sample,
                           ifelse(obj$paired,"Paired-","One-"),
			   "Two-"),
                "Sample Bootstrap Confidence Interval",
		sep="")
 cat(title, "\n")
 cat(paste(rep("=",nchar(title)),collapse=""), "\n\n")

 if (length(data.<-obj$data.)==1)
  cat("Data:", data., "\n")
 else
  for(i in 1:2)
    cat("Sample", paste(i,":",sep="") , data.[i], "\n")

 if (obj$one.sample)
  {parameter <- if (obj$paired)
		 "mu[D]"
		else
                 switch(obj$parameter,
		        mean="mu",
		        variance="sigma^2",
		        proportion="p")
   if (obj$log.)
    parameter <- paste("log(", parameter, ")", sep="")
  }
 else
  {comparison <- obj$comparison
   parameter <- switch(obj$parameter,
		       mean="mu",
		       variance="sigma",
		       proportion="p"
                       )
   parameter <- paste(parameter, "[", 1:2, "]", sep="")
   if (obj$parameter=="variance")
    parameter <- paste(parameter, 2, sep="^")
   if (obj$log.) 
    {parameter <- paste("log(", parameter, ")", sep="")
     comparison <- "difference"
    }
   op <- switch(comparison, difference=" - ", ratio=" / ")
   parameter <- paste(parameter,
                      collapse=op)
  }
 cat("Parameter of interest:", parameter, "\n")
 cat("Bootstrap estimate:", obj$estimate, "\n")
 cat("Bootstrap standard error:", obj$se, "\n")
 if (percent)
  {perci <- obj$perci
   perci <- matrix(perci, 1, 2,
              dimnames=list("Bootstrap percentile c.i.:",names(perci)))
   cat("\n")
   print(perci)
  }
 if (t.)
  {tci <- obj$tci
   tci <- matrix(tci, 1, 2,
              dimnames=list("Bootstrap t c.i.:", names(tci)))
   cat("\n")
   print(tci)
  }
 cat("\nNote:   Bootstrap size =", obj$boot.size, "\n")
 invisible()
}
