##### Permutation/Randomization Test #####
## R functions:
##     compare: generate compare object for the test results of data
##     print.compare: print method for compare
##     plot.compare: plot method for compare
##     summary.compare: summary method for compare
compare <- function(x, y, m=length(x), n=length(y),
           comparison=c("mean.diff","variance.ratio",
                        "prop.diff","ratio.prop"),
           alternative=c("two.sided","less","greater"),
	   null.val=NULL, log.=FALSE,
	   method=c("randomization","permutation","bootstrap"),
	   size=1000, Max.size=10000)
# Created by J.C. Wang, Feb. 21, 2008
{nobs <- m + n

 method <- match.arg(method)
 if (method=="permutation")
  size <- choose(nobs, m)

 if (size > Max.size) {
  cat("Requested size", size, "is too large\n")
  size <- Max.size
  cat("\t... reset to", size, "\n")
  if (method=="permutation") {
   cat("Requested permutation test is changed to",
       "randomization test!\n")
   method <- "randomization"
  }
 }

 samples <- c(deparse(substitute(x)), deparse(substitute(y)))
 snames <- c("x","y");  alphanumeric <- as.character(0:9)
 for(i in 1:2)
  if (substr(samples[i],1,1) %in% alphanumeric)
   samples[i] <- snames[i]

 comparison <- match.arg(comparison)

 props <- c("prop.diff", "ratio.prop")
 if (length(x)==1) {
  stopifnot(x<=m, comparison %in% props)
  x <- rep(1:0,c(x,m-x))
 }
 if (length(y)==1) {
  stopifnot(y<=n, comparison %in% props)
  y <- rep(1:0,c(y,n-y))
 }

 parameter <- unlist(strsplit(comparison, "\\."))
 comp <- parameter[2]
 parameter <- parameter[1]
 if (parameter=="ratio"){
  parameter <- "prop"
  comp <- "ratio"
 }

 if (is.null(null.val))
   null.val <- switch(comp, diff=0, ratio=1)

 if (log. && (parameter!="variance")) {
  cat("log.=TRUE ignored and reset to FALSE\n")
  log. <- FALSE
 }

 est <- function(x,theta) 
         ifelse(theta=="variance", var(x), mean(x))
 stat <- function(x,y,theta,comp) {
	  e1 <- est(x,theta)
	  e2 <- est(y,theta)
	  switch(comp, diff=e1-e2, ratio=e1/e2)
         }

 statistic <- stat(x, y, parameter, comp)
 if (log.) statistic <- log(statistic)

 xs <- 1:m
 data <- c(x,y)
 if(method=="randomization")
  {ans <- numeric(size)
   for(i in 1:size){
    new.x <- sample(data)
    new.y <- new.x[-xs]
    new.x <- new.x[xs]
    ans[i] <- stat(new.x, new.y, parameter, comp)
   }
  }
  else if (method=="permutation")
  {f <- function(x, data, x.small, param, comp)
          {if (x.small) stat(data[x], data[-x], param, comp)
	    else stat(data[-x], data[x], param, comp)
	  }
   ans <- combn(nobs, min(m,n), FUN=f, data=data, x.small=(m <= n),
                param=parameter, comp=comp)
   ans <- as.vector(ans) 
  }
  else   # if (method=="bootstrap")
  {ans <- numeric(size)
   for(i in 1:size){
    new.x <- sample(data, nobs, replace=T)
    new.y <- new.x[-xs]
    new.x <- new.x[xs]
    ans[i] <- stat(new.x, new.y, parameter, comp)
   }
  }
 if (log.) ans <- log(ans)

 alternative <- match.arg(alternative)

 pval <- switch(alternative, 
	  less=mean((ans<statistic)+(ans==statistic)/2),
	  greater=mean((ans>statistic)+(ans==statistic)/2),
	  two.sided={ stype <- if (log.) "diff" else comp
	   right <- switch(stype,
	              diff=abs(statistic),
	              ratio=ifelse(statistic>1, statistic,
		                                1/statistic))
	   left <- switch(stype, diff= -right, ratio=1/right)
	   mean((ans>right)+(ans<left)+(ans %in% c(left,right))/2)
	  }
         )

 parameter <- switch(parameter, mean="mu",
                                variance="sigma",
				prop="p")
 parameter <- paste(parameter,"[",1:2,"]",
		    if(parameter=="sigma") "^2" else "",
                    sep="")
 parameter <- paste(parameter,
                    collapse=switch(comp,diff="-",ratio="/"))

 if (comp=="diff") comp <- "difference"

 out <- list(parameter=parameter, comparison=comp,
             alternative=alternative, null.val=null.val,
	     method=method, statistic=statistic, p.val=pval,
	     log. = log., size=size, samples=samples, reference=ans)
 class(out) <- "compare"

 out
}


print.compare <- function(obj, reference=F)
# Created by J.C. Wang, Feb. 21, 2008
{if (!reference) obj <- obj[-match("reference",names(obj))]
 print(unclass(obj))
}


summary.compare <- function(obj)
# Created by J.C. Wang, Feb. 21, 2008
{samples <- paste("Sample ", 1:2, ": ", obj$samples, sep="")
 samples <- paste(samples, collapse="\n")

 hypotheses <- paste("H", 0:1, ": ", obj$parameter, 
		     c("=",
		       switch(obj$alternative,
		              less="<",
			      greater=">",
			      two.sided="!=")),
		       obj$null.val,
		       sep="")
 hypotheses <- paste(hypotheses, collapse="  vs.  ")

 cat("Two Samples Comparison\n")
 cat("======================\n\n")
 cat(samples, "\n")
 cat("Comparison:", obj$parameter, "\n")
 cat(hypotheses, "\n")
 cat("Method used:", obj$method, "test\n\n")
 cat("Test statistic:", obj$statistic, "\n")
 cat("P value:", obj$p.val, "\n")
 if (obj$log.)
   cat("Note: Test statistic is log-ratio of sample variances.\n")
 if (obj$method=="randomization")
   cat("\n(based on", obj$size, "random permutations)\n")
  else if (obj$method=="permutation")
   cat("\n(based on", obj$size, "all possible permutations)\n")
  else  # if (obj$method=="bootstrap")
   cat("\n(based on", obj$size, "bootstrap samples)\n")
 cat("\n\n")
 invisible()
}

plot.compare <- function(obj, breaks="Sturges",
		freq=FALSE, col="wheat", border=NULL,
                xlab=paste(ifelse(obj$log.,"log ",""),
		           obj$comparison, sep=""),
		ylab="density",
		main="Two Samples Comparison",
		density.=FALSE, adjust=2, dcol="blue", dlwd=1,
		pch.=24, stat.col="red", bg.="green3",
		digits=5,
                ...)
# Created by J.C. Wang, Feb. 21, 2008
{old.pars <- par(mar=c(5,4,4,1)+.1)
 on.exit(par(old.pars))

 hist(obj$reference, breaks, freq, col=col, border=border,
      xlab=xlab, ylab=ylab, main=main, ...)

 if (density.)
  lines(density(obj$reference, adjust=adjust),
        col=dcol, lwd=dlwd)

 box()

 hypotheses <- paste("H[", 0:1, "] * ': ' * ", obj$parameter, 
		     c("==",
		       switch(obj$alternative,
		              less="<",
			      greater=">",
			      two.sided="!=")),
		       rep(obj$null.val,2),
		       sep="")
 mtext(side=3, line=0.2, adj=0, text=parse(text=hypotheses[1]))
 mtext(side=3, line=0.2, text="versus")
 mtext(side=3, line=0.2, adj=1, text=parse(text=hypotheses[2]))

 mtext(side=4, line=0.05,
       text=paste("(based on ",obj$size,
		  switch(obj$method,
		         randomization=" random permutations",
		         permutation=" all possible permutations",
			 bootstrap=" bootstrap samples"),
                  ")",sep=""))

 line. <- par('mar')[1] - 1.1
 for(i in 1:2)
  mtext(side=1, line=line., adj=i-1,
       text=paste("Sample ",i,": ",obj$samples[i],sep=""))

 usr <- par('usr')
 points(obj$statistic, mean(c(usr[3],0)),
        pch=pch., col=stat.col, bg=bg.)

 stat <- paste("  statistic=", signif(obj$statistic,digits),
               " ", sep="")
 w <- strwidth(stat)
 h <- strheight(stat)
 text(usr[2], usr[4]-h, label=stat, adj=1)
 points(usr[2]-w, usr[4]-1.2*h, pch=pch., col=stat.col, bg=bg.)

 pval <- paste("P Value=", signif(obj$p.val,digits), " ", sep="")
 text(usr[2], usr[4]-2.5*h, label=pval, adj=1)
}
